#ifndef _GRAPH_SUP_H
#define _GRAPH_SUP_H

#define WIDTH 1600
#define HEIGHT 300

struct Button {
	int x;
	int y;
	int w;
	int h;
	int id;
};

typedef struct Button button;

extern button* Buttons;
extern int BT_Count;

void ClearButtons();
int GetButton(int x, int y);
void AddButton(int x, int y, int w, int h, int id);

int DrawNextFrame(void);

couleur GetColorN(char* name);
couleur GetColor(unsigned char r, unsigned char g, unsigned char b);
void SetColor(unsigned char r, unsigned char g, unsigned char b);
void SetColorC(couleur Color);
void SetColorN(char* name);

#endif